@@ -6,7 +6,7 @@ package BSON::OID;
 # ABSTRACT: BSON type wrapper for Object IDs
 
 use version;
-our $VERSION = 'v1.6.0';
+our $VERSION = 'v1.6.1';
 
 use Carp;
 use Config;
@@ -51,7 +51,7 @@ use namespace::clean -except => 'meta';
     sub _packed_oid {
         my $time = defined $_[0] ? $_[0] : time;
         return pack(
-            'Na3na3', $time, $_host, $$ % 0xFFFF,
+            'Na3na3', $time, $_host, $$ & 0xFFFF,
             substr( pack( 'N', do { lock($_inc); $_inc++; $_inc %= 0xFFFFFF }), 1, 3)
         );
     }
@@ -244,7 +244,7 @@ BSON::OID - BSON type wrapper for Object IDs
 
 =head1 VERSION
 
-version v1.6.0
+version v1.6.1
 
 =head1 SYNOPSIS
 
