@@ -6,18 +6,20 @@ package BSON::PP;
 # ABSTRACT: Pure Perl BSON implementation
 
 use version;
-our $VERSION = 'v1.6.0';
+our $VERSION = 'v1.6.1';
 
 use B;
 use Carp;
 use Config;
 use Scalar::Util qw/blessed looks_like_number refaddr reftype/;
+use List::Util qw/first/;
 use Tie::IxHash;
 
 use BSON::Types ();
 use boolean;
+use mro;
 
-use if $] ge '5.010000', 're', 'regexp_pattern';
+use re 'regexp_pattern';
 
 use constant {
     HAS_INT64 => $Config{use64bitint},
@@ -91,6 +93,18 @@ sub _ixhash_iterator {
     }
 }
 
+# relying on Perl's each() is prone to action-at-a-distance effects we
+# want to avoid, so we construct our own iterator for hashes
+sub _hashlike_iterator {
+    my $hashlike = shift;
+    my @keys = keys %$hashlike;
+    return sub {
+        my $k = shift @keys;
+        return unless defined $k;
+        return ($k, $hashlike->{$k});
+    }
+}
+
 # XXX could be optimized down to only one substr to trim/pad
 sub _bigint_to_int64 {
     my $bigint = shift;
@@ -182,6 +196,8 @@ sub _encode_bson {
       : $doc_type eq 'MongoDB::DBRef' ? _ixhash_iterator( $doc->_ordered )
       :                                 do { _reftype_check($doc); undef };
 
+    $iter //= _hashlike_iterator($doc);
+
     my $op_char = defined($opt->{op_char}) ? $opt->{op_char} : '';
     my $invalid =
       length( $opt->{invalid_chars} ) ? qr/[\Q$opt->{invalid_chars}\E]/ : undef;
@@ -192,7 +208,7 @@ sub _encode_bson {
     my $bson = '';
 
     my ($key, $value);
-    while ( $first_key_pending or ( $key, $value ) = $iter ? $iter->() : (each %$doc) ) {
+    while ( $first_key_pending or ( $key, $value ) = $iter->() ) {
         next if defined $first_key && $key eq $first_key;
 
         if ( $first_key_pending ) {
@@ -222,6 +238,12 @@ sub _encode_bson {
         utf8::encode($utf8_key);
         my $type = ref $value;
 
+        # If the type is a subtype of BSON::*, use that instead
+        if ( blessed $value ) {
+            my $parent = first { /\ABSON::\w+\z/ } reverse @{mro::get_linear_isa($type)};
+            $type = $parent if defined $parent;
+        }
+
         # Null
         if ( !defined $value ) {
             $bson .= pack( BSON_TYPE_NAME, 0x0A, $utf8_key );
@@ -403,46 +425,16 @@ sub _encode_bson {
             }
         }
 
-        ### SCALAR: numbers preferred
-
-        elsif ( $opt->{prefer_numeric} ) {
-
-            # Int heuristic based on size
-            if ( $value =~ $int_re ) {
-                if ( $value > $max_int64 || $value < $min_int64 ) {
-                    croak("BSON can only handle 8-byte integers. Key '$key' is '$value'");
-                }
-                elsif ( $value > $max_int32 || $value < $min_int32 ) {
-                    $bson .= pack( BSON_TYPE_NAME, 0x12, $utf8_key ) . _pack_int64($value);
-                }
-                else {
-                    $bson .= pack( BSON_TYPE_NAME . BSON_INT32, 0x10, $utf8_key, $value );
-                }
-            }
-
-            # Double
-            elsif ( $value =~ $doub_re ) {
-                $bson .= pack( BSON_TYPE_NAME.BSON_DOUBLE, 0x01, $utf8_key, $value );
-            }
-
-            # String
-            else {
-                utf8::encode($value);
-                $bson .= pack( BSON_TYPE_NAME.BSON_STRING, 0x02, $utf8_key, $value );
-            }
-
-        }
-
-        ### SCALAR: strings preferred
-
+        # SCALAR
         else {
+            # If a numeric value exists based on internal flags, use it;
+            # otherwise, if prefer_numeric is true and it looks like a
+            # number, then coerce to a number of the right type;
+            # otherwise, leave it as a string
 
             my $flags = B::svref_2object(\$value)->FLAGS;
-            if ( $flags & B::SVf_POK() ) {
-                utf8::encode($value);
-                $bson .= pack( BSON_TYPE_NAME.BSON_STRING, 0x02, $utf8_key, $value );
-            }
-            elsif ( $flags & B::SVf_NOK() ) {
+
+            if ( $flags & B::SVf_NOK() ) {
                 $bson .= pack( BSON_TYPE_NAME.BSON_DOUBLE, 0x01, $utf8_key, $value );
             }
             elsif ( $flags & B::SVf_IOK() ) {
@@ -456,23 +448,37 @@ sub _encode_bson {
                     $bson .= pack( BSON_TYPE_NAME . BSON_INT32, 0x10, $utf8_key, $value );
                 }
             }
-            elsif ( $] lt '5.010' && $flags & B::SVp_IOK() ) {
-                if ( $value > $max_int64 || $value < $min_int64 ) {
-                    croak("BSON can only handle 8-byte integers. Key '$key' is '$value'");
+            elsif ( $opt->{prefer_numeric} && looks_like_number($value) ) {
+                # Looks like int: type heuristic based on size
+                if ( $value =~ $int_re ) {
+                    if ( $value > $max_int64 || $value < $min_int64 ) {
+                        croak("BSON can only handle 8-byte integers. Key '$key' is '$value'");
+                    }
+                    elsif ( $value > $max_int32 || $value < $min_int32 ) {
+                        $bson .= pack( BSON_TYPE_NAME, 0x12, $utf8_key ) . _pack_int64($value);
+                    }
+                    else {
+                        $bson .= pack( BSON_TYPE_NAME . BSON_INT32, 0x10, $utf8_key, $value );
+                    }
                 }
-                elsif ( $value > $max_int32 || $value < $min_int32 ) {
-                    $bson .= pack( BSON_TYPE_NAME, 0x12, $utf8_key ) . _pack_int64($value);
+
+                # Looks like double
+                elsif ( $value =~ $doub_re ) {
+                    $bson .= pack( BSON_TYPE_NAME.BSON_DOUBLE, 0x01, $utf8_key, $value );
                 }
+
+                # looks_like_number true, but doesn't match int/double
+                # regexes, so as a last resort we leave as string
                 else {
-                    $bson .= pack( BSON_TYPE_NAME . BSON_INT32, 0x10, $utf8_key, $value );
+                    utf8::encode($value);
+                    $bson .= pack( BSON_TYPE_NAME.BSON_STRING, 0x02, $utf8_key, $value );
                 }
             }
             else {
-                # Last resort, string
+                # Not coercing or didn't look like a number
                 utf8::encode($value);
                 $bson .= pack( BSON_TYPE_NAME.BSON_STRING, 0x02, $utf8_key, $value );
             }
-
         }
     }
 
@@ -806,7 +812,7 @@ BSON::PP - Pure Perl BSON implementation
 
 =head1 VERSION
 
-version v1.6.0
+version v1.6.1
 
 =head1 DESCRIPTION
 
